# Log.py
import os
import re
import time

LEN_CACHE = 10000 # max lines of log header
LEN_CACHE_BYTES = 100*1024 # max lines of log header

CACHE_HEADER = None
CACHE_TAIL = None

class Log():

    def __init__(self, path, case):
        self.path = path
        self.case = case
        if self.path:
            self.cached_header =  self.cache_header()
            self.cached_body = self.cache_body()
            self.log = True
        else:
            self.log = False

    @property
    def is_valid(self):
        # TODO Fails on decompose logs
        if not self.path:
            return False
        if (self.Exec == "decomposePar") or (self.Exec == "blockMesh") or (self.Exec == "mapFields"):
            return False
        try:
            self.get_SimTime(self.cached_body)
            return True
        except Exception as e:
            # print("Invalid Log", e)
            return False

    @property
    def Exec(self):
        return self.get_header_value("exec")

    @property
    def nProcs(self):
        return self.get_header_value("nProcs")

    @property
    def Host(self):
        return self.get_header_value("Host")

    @property
    def build(self):
        return self.get_header_value("build")

    @property
    def Case(self):
        return self.get_header_value("Case")

    @property
    def PID(self):
        return self.get_header_value("PID")

    @property
    def getctime(self):
        return os.path.getctime(self.path)

    @property
    def active(self):
        if not self.log:
            return False
        return (time.time() - self.getctime) < 60.0

    def cache_header(self):
        """ read LEN_HEADER lines from log """
        with open(self.path, "rb") as fh:
            header = fh.read(LEN_CACHE_BYTES).decode('utf-8')
            ctime = header.find("ClockTime")
            padding = min(ctime+100, len(header))
            header = header[0:padding] # use 100 padding chars
            return header #.split("\n")

    def cache_body(self):
        """ read LEN_HEADER lines from log """
        with open(self.path, "rb") as fh:
            fh.seek(fh.tell(), os.SEEK_END)
            fh.seek(max(0, fh.tell()-LEN_CACHE_BYTES), os.SEEK_SET)
            return fh.read(LEN_CACHE_BYTES).decode('utf-8') #.split("\n")

    def refresh(self):
        self.cached_body = self.cache_body()

    def get_ClockTime(self, chunk, last=True):
        try:
            return float(re.findall("ClockTime = ([0-9.]*) s", chunk)[-1])
        except:
            return 0.0


    def get_SimTime(self, chunk):
        try:
            return float(re.findall("\nTime = ([0-9.e\-]*)", chunk)[-1])
        except:
            return 0.0

    def get_header_value(self, key):
        ret =  re.findall("{: <7}: ([0-9A-Za-z]*)".format(key), self.cached_header)
        if ret:
            return ret[0]
        else:
            return None

    def text(self, filter_):
        lines = self.cache_body().split("\n")
        if filter_:
            return "\n".join([l for l in lines if filter_ in l])
        else:
            return "\n".join(lines)

    def print_log_body(self):
        sep_width = 120
        print(self.path)
        print("="*sep_width)
        if self.case.log_filter:
            lines = self.cached_body.split("\n")
            filt_lines = [l for l in lines if self.case.log_filter in l][-30:-1]
            body_str = ("\n".join(filt_lines))
        else:
            body_str = ("\n".join(self.cached_body.split("\n")[-30:-1]))
        print(body_str)

    @property
    def start_time(self):
        return self.get_SimTime(self.cached_header)

    @property
    def sim_time(self):
        return self.get_SimTime(self.cached_body)

    @property
    def wall_time(self):
        return self.get_ClockTime(self.cached_body)

    def remaining_sim_time(self, endtime):
        return endtime - self.sim_time

    @property
    def elapsed_sim_time(self):
        return self.sim_time - self.start_time

    def progress(self, endtime):
        return (self.sim_time)/endtime

    @property
    def sim_speed(self):
        return max(1e-12, self.elapsed_sim_time/max(1.0, self.wall_time))

    @property
    def is_parallel(self):
        # TODO use regex
        for line in self.cached_header.split("\n"):
            if 'Exec' in line and "-parallel" in line:
                    return True
        return False

        return max(1e-12, self.elapsed_sim_time/self.wall_time)

    def timeleft(self):
        return self.time_till(self.case.endTime)

    def time_till_writeout(self):
        return self.time_till(
                self.case.last_timestep
              + self.case.writeInterval)

    def time_till(self, end):
        return self.remaining_sim_time(end)/(self.sim_speed)





from docopt import docopt
from colorama import Fore, Back, Style
# from datetime import datetime, timedelta
import datetime
import os
import time
from concurrent.futures import ThreadPoolExecutor
from collections import defaultdict
from copy import deepcopy
from .Log import Log
from .header import foamMonHeader

import sys


def timedelta(seconds):
    return datetime.timedelta(seconds=int(max(0, seconds)))

class ColoredProgressBar():


    events = []

    def __init__(self, size, progress=0):
        self.size = size
        self.done_char = Fore.GREEN + "█" + Style.RESET_ALL
        self.undone_char = Fore.BLUE + "█" + Style.RESET_ALL
        self.digits = [self.done_char
                for _ in range(int(progress*size))]
        self.digits.extend([self.undone_char
                for _ in range(size - int(progress*size))])


    def add_event(self, percentage, color):
        index = int(percentage*self.size)
        self.digits[index] = color + "█" + Style.RESET_ALL

    def draw(self):
        return "".join(self.digits)

class ProgressBar():


    events = []

    def __init__(self, size, progress=0):
        self.size = size
        self.done_char = "█"
        self.undone_char = "░"
        self.digits = [self.done_char
                for _ in range(int(progress*size))]
        self.digits.extend([self.undone_char
                for _ in range(size - int(progress*size))])


    def add_event(self, percentage, color):
        index = int(percentage*self.size)
        self.digits[index] = "█"

    def draw(self):
        return "".join(self.digits)


class Cases():

    def __init__(self, path):
        self.path = path
        os.system("clear")
        self.mdates = {}
        self.cases = defaultdict(list)
        p = ThreadPoolExecutor(1)
        self.running = True
        p.submit(self.find_cases)

    def get_valid_cases(self):
        case_stats = {}
        cases = deepcopy(self.cases)
        for r, cs in cases.items():
            for c in cs:
                if (c.log.active):
                    c.log.refresh()
            case_stats[r] = {"active": [c.print_status_short() for c in cs
                    if (c.print_status_short() and c.log.active)],
                        "inactive": [c.print_status_short() for c in cs
                    if (c.print_status_short() and not c.log.active)]
                    }
        lengths = self.get_max_lengths(case_stats)
        return lengths, case_stats

    def get_max_lengths(self, statuses):
        lengths = [0 for _ in range(6)]
        for n, folder in statuses.items():
            for s in folder.get("active", []):
                for i in range(len(lengths)):
                    lengths[i] = max(lengths[i], s.lengths[i])

            for s in folder.get("inactive", []):
                for i in range(len(lengths)):
                    lengths[i] = max(lengths[i], s.lengths[i])
        return lengths

    def find_cases(self):

        while self.running:
            # TODO store modification dates and traverse only
            # on updated folder

            top = self.path

            c = Case(top)
            subfold = top.split("/")[-1]
            if c.is_valid:
                exists = False
                for existing in self.cases[subfold]:
                    if c.path == existing.path:
                        exists = True
                if not exists:
                    self.cases[subfold].append(c)

            root_mdate = False

            # rescan only if not scanned before or folder has changed
            if root_mdate and os.path.getmtime(top) <= root_mdate:
                # wait 10 seconds
                time.sleep(10)
                continue

            root_mdate = os.path.getmtime(top)

            for r, dirs, _ in os.walk(self.path):

                ignore = [
                    "boundaryData",
                    "uniform",
                    "processor",
                    "constant",
                    "lagrangian",
                    "postProcessing",
                    "dynamicCode",
                    "system"]

                for d in deepcopy(dirs):
                    for i in ignore:
                        if d.startswith(i):
                            dirs.remove(d)
                    full_path =  os.path.join(r, d)
                    last_mdate = self.mdates.get(full_path)
                    if last_mdate and last_mdate >= os.path.getmtime(full_path):
                        dirs.remove(d)

                for d in dirs:
                    try:
                        c = Case(os.path.join(r, d))
                        subfold = r.split("/")[-1]
                        if c.is_valid:
                            exists = False
                            for existing in self.cases[subfold]:
                                if c.path == existing.path:
                                    exists = True
                            if not exists:
                                full_path =  os.path.join(r, d)
                                self.mdates[full_path] = os.path.getmtime(full_path)
                                self.cases[subfold].append(c)
                    except Exception as e:
                        print("innner", e, r, d)
                        pass
                time.sleep(30)

    def print_header(self, lengths):
        width_progress = lengths[0]
        width_folder =  lengths[1] + 2
        width_log =  lengths[2] + 2
        width_time =  lengths[3] + 2
        width_next_write =  max(12, lengths[4] + 2)
        width_finishes =  lengths[5] + 2
        s = "  {: ^{width_progress}}|{: ^{width_folder}}|{: ^{width_log}}|"
        s +="{: ^{width_time}}|{: ^{width_next_write}}|{: ^{width_finishes}}"
        s = s.format("Progress", "Folder", "Logfile", "Time", "Next write", "Finishes",
                width_progress=width_progress,
                width_folder=width_folder,
                width_log=width_log,
                width_time=width_time,
                width_next_write=width_next_write,
                width_finishes=width_finishes,
                )
        print(s)

    def print_legend(self):
        s = "\nLegend: "
        s += Fore.GREEN + "█" + Style.RESET_ALL + " Progress "
        s += Fore.YELLOW + "█"  + Style.RESET_ALL + " Start Sampling "
        s += Style.BRIGHT + "Active"  + Style.RESET_ALL  + " "
        s += Style.DIM + "Inactive"  + Style.RESET_ALL + "\n"
        print(s)

    def print_status(self):
        str_ = "\n".join([c.print_status_short()
            for c in self.cases
            if (c.print_status_short and c.log.active)])
        print(str_)


class Case():

    def __init__(self, path, log_format="log", summary=False, log_filter=None):
        self.path = path
        self.basename = os.path.basename(self.path)
        self.log_format = log_format
        self.log_fns = self.find_logs(self.log_format)
        self.log = Log(self.find_recent_log_fn(), self)
        self.log_filter = log_filter

        if summary:
            if self.log.active:
                ret = self.print_status_short()
                if ret:
                    print(ret)

    @property
    def is_valid(self):
        # print("check if valid", self.path)
        return self.has_controlDict and self.log.is_valid


    @property
    def started_sampling(self):
        return self.simTime > self.startSampling

    @property
    def has_controlDict(self):
        ctDct =  os.path.exists(self.controlDict_file)
        return ctDct

    def status_bar(self, digits=100):
        bar = ProgressBar(digits, self.log.progress(self.endTime))
        bar.add_event(self.startSamplingPerc, Fore.YELLOW)
        return bar.draw()


    def find_logs(self, log_format):
       """ returns a list of filenames and ctimes """
       # print(self.path)
       r, d, files = next(os.walk(self.path))
       # TODO use regex to find logs
       files = list(filter(lambda x: log_format in x, files))
       files = [os.path.join(r, f) for f in files]
       ctimes = [os.path.getctime(os.path.join(self.path, f)) for f in files]
       # print(self.path, files)
       return list(zip(ctimes, files))

    @property
    def last_timestep(self):
        if self.log.is_parallel:
            proc_dir = os.path.join(self.path, "processor0")
            if not os.path.exists(proc_dir):
                return 0
            r, ds, _ = next(os.walk(proc_dir))
            ds = [float(d) for d in ds if "constant" not in d]
            if ds:
                return max(ds)
            else:
                return 0
        else:
            ts = []
            r, ds, _ = next(os.walk(self.path))
            for t in ds:
                try:
                    tsf = float(t)
                    ts.append(tsf)
                except:
                    pass
            if ts:
                return max(ts)
            else:
                return 0.0

    def find_recent_log_fn(self):
        try:
            ctimes, files = zip(*self.log_fns)
            latest_index = ctimes.index(max(ctimes))
            return files[latest_index]
        except:
            return False

    @property
    def controlDict_file(self):
        return os.path.join(self.path, "system/controlDict")

    def get_key_controlDict(self, key):
        """ find given key in controlDict """
        # TODO cach controlDict to avoid reopening file
        separator = " "
        key += separator
        if self.has_controlDict:
            with open(self.controlDict_file) as f:
                for i, line in enumerate(f.readlines()):
                    if key in line:
                        # TODO use regex to sanitise
                        return (line.replace(key, '')
                                .replace(' ', '')
                                .replace(';', '')
                                .replace('\n', ''))
        else:
            return None

    def get_float_controlDict(self, key):
        ret = self.get_key_controlDict(key)
        if ret:
            return float(ret)
        else:
            return 0

    @property
    def endTime(self):
        return self.get_float_controlDict("endTime")

    @property
    def writeControl(self):
        return self.get_key_controlDict("writeControl")

    @property
    def writeInterval(self):
        if self.writeControl == "runTime":
            return self.get_float_controlDict("writeInterval")
        else:
            return (self.get_float_controlDict("writeInterval") *
                    self.get_float_controlDict("deltaT"))

    @property
    def startSampling(self):
        return self.get_float_controlDict("timeStart")

    @property
    def startSamplingPerc(self):
        return self.startSampling/self.endTime

    @property
    def simTime(self):
        return self.log.sim_time

    def print_status_short(self):
        try:
            exc_info = sys.exc_info()
            return Status(
                    self,
                    self.log.progress(self.endTime),
                    # Style.BRIGHT if self.log.active else Style.DIM,
                    50,
                    self.log.active,
                    self.basename,
                    os.path.basename(self.log.path),
                    self.log.sim_time,
                    timedelta(self.log.time_till_writeout()),
                    timedelta(self.log.timeleft()),
                    # Style.RESET_ALL
                    )
        except Exception as e:
            import traceback
            print(e, self.path)
            print(traceback.print_exception(*exc_info))
            return False

    def print_status_full(self):
        while True:
            self.log.print_log_body()
            try:
                prog_prec = self.log.progress(self.endTime)*100.0
                print(self.status_bar(100))
                print("Case properties: ")
                print("Job start time: ", self.log.start_time)
                print("Job elapsed_ time: ", timedelta(seconds=self.log.wall_time))
                print("Active: ", self.log.active)
                print("Parallel: ", self.log.is_parallel)
                print("Case end time: ", self.endTime)
                print("Current sim time: ", self.log.sim_time)
                print("Last time step on disk: ", self.last_timestep)
                print("Time next writeout: ", timedelta(self.log.time_till_writeout()))
                print("Progress: ", prog_prec)
                print("Timeleft: ", timedelta(self.log.timeleft()))
            except Exception as e:
                print(e)
                pass
            self.log.refresh()
            time.sleep(0.2)
            os.system("clear")


class Status():
    """ Handle status of single case for simple printing  """

    def __init__(self, case, progress, digits, active, base, name, time, wo, tl):
        self.case = case
        self.progress = progress
        self.digits = digits
        self.active = active
        self.base = base
        self.name = name
        self.time = str(time)
        self.wo = str(wo)
        self.tl = str(tl)

    @property
    def lengths(self):
        return [self.digits, len(self.base), len(self.name),
                len(self.time), len(self.wo), len(self.tl)]



import subprocess
import os

from setuptools import setup

version_py = os.path.join(os.path.dirname(__file__), 'FoamMon/version.py')

try:
    version_git = subprocess.check_output(["git", "describe"], universal_newlines=True).rstrip()
except:
    with open(version_py, 'w') as fh:
        version_git = open(version_py).read().strip().split('=')[-1].replace('"','')

version_msg = "# Do not edit this file, pipeline versioning is governed by git tags"
with open(version_py, 'w') as fh:
    fh.write(version_msg + os.linesep + "__version__='{}'".format(version_git))

package_name = 'FoamMon'
config = {
    'author'                 : 'Gregor Olenik',
    'author_email'           : 'go@hpsim.de',
    'description'            : '',
    'license'                : 'MIT',
    'version'                : version_git,
    'packages'               : ["FoamMon"],
    'install_requires'       : [
         'docopt',
         'colorama',
         'urwid',
    ],
   'name'                    : 'foamMon',
   'scripts': ["bin/foamMon"]}


setup(**config)



import urwid

from .header import foamMonHeader
import os
from .FoamDataStructures import Cases
import threading
import time
import datetime

# Set up color scheme
palette = [
    ('titlebar', 'dark red', ''),
    ('refresh button', 'dark green,bold', ''),
    ('progress', '', 'dark green'),
    ('unprogressed', '', 'dark blue'),
    ('sampling', '', 'yellow'),
    ('quit button', 'dark red,bold', ''),
    ('getting quote', 'dark blue', ''),
    ('active', 'white,bold', ''),
    ('mode button', 'white,bold', ''),
    ('inactive ', 'light gray', ''),
    ('change negative', 'dark red', '')]


CASE_CTR = 0
CASE_REFS = {}
MODE_SWITCH = False
FOCUS_ID = None
FILTER = None


class ProgressBar():


    events = []

    def __init__(self, size, progress=0):
        self.size = size
        self.done_char = ("progress", " ")  # "█"
        self.undone_char = ("unprogressed", " ")  #"█"
        self.digits_done = [self.done_char
                for _ in range(int(progress*size))]
        self.digits_undone = [self.undone_char
                for _ in range(size - int(progress*size))]
        self.digits = self.digits_done + self.digits_undone


    def add_event(self, percentage, color):
        index = int(percentage*self.size)
        self.digits[index] = (color, " ")

    def draw(self):
        return "".join(self.digits)

    def render(self):
        return urwid.Text(self.digits)


class CaseRow(urwid.WidgetWrap):

    def __init__(self, case, Id, length=False, active=False):

        self.case = case
        self.active = active
        self.lengths = length
        global CASE_CTR
        global CASE_REFS
        CASE_CTR +=  1
        self.Id = CASE_CTR
        CASE_REFS[int(self.Id)] = self.case.case

        mode_text = "active" if self.active else "inactive"

        bar = ProgressBar(50, self.case.progress)
        bar.add_event(self.case.case.startSamplingPerc, "sampling")
        bar = bar.render()
        urwid.WidgetWrap.__init__(self, urwid.Columns(
            [("pack", urwid.Text((mode_text, "{: ^2} ".format(self.Id)))),
                ("pack", bar),
                ("pack", urwid.Text((mode_text, self.status_text)))]))

    @property
    def status_text(self):
        width_folder =  self.lengths[1] + 2
        width_log =  self.lengths[2] + 2
        width_time =  self.lengths[3] + 2
        width_next_write =  max(12, self.lengths[4] + 2)
        width_finishes =  self.lengths[5] + 2
        # s = "{: ^{width_progress}}│"
        s = "{: ^{width_folder}}{: ^{width_log}}"
        s += "{: ^{width_time}}{: ^{width_next_write}}{: ^{width_finishes}}"
        s = s.format(
                # index,
                self.case.base, self.case.name,
                self.case.time, self.case.wo, self.case.tl,
                # width_progress=width_progress,
                width_folder=width_folder,
                width_log=width_log,
                width_time=width_time,
                width_next_write=width_next_write,
                width_finishes=width_finishes,
                )
        return s

class TableHeader():

    def __init__(self, lengths):
        self.lengths = lengths

    @property
    def header_text(self):
        width_folder =  self.lengths[1] + 2
        width_log =  self.lengths[2] + 2
        width_time =  self.lengths[3] + 2
        width_next_write =  max(12, self.lengths[4] + 2)
        width_finishes =  self.lengths[5] + 2
        # s = "{: ^{width_progress}}│"
        s = "{: ^{width_progress}}{: ^{width_folder}}{: ^{width_log}}"
        s += "{: ^{width_time}}{: ^{width_next_write}}{: ^{width_finishes}}"
        s = s.format(
                "Progress",
                "Folder", "Logfile",
                "Time", "Writeout", "Remaining",
                # width_progress=width_progress,
                width_progress=54,
                width_folder=width_folder,
                width_log=width_log,
                width_time=width_time,
                width_next_write=width_next_write,
                width_finishes=width_finishes,
                )
        return s


class DisplaySub(urwid.WidgetWrap):

    def __init__(self, Id, name, elems, lengths, hide_inactive):
        self.path = name
        self.elems = elems
        self.lengths = lengths
        self.Id = Id
        self.hide_inactive = hide_inactive
        self.frame = self.draw()
        urwid.WidgetWrap.__init__(self, self.frame)

    @property
    def active(self):
        return self.elems["active"]

    @property
    def inactive(self):
        return self.elems["inactive"]

    def draw(self):
        items = [("pack", CaseRow(c, (i+1)*self.Id, self.lengths, active=True))
                for i, c in enumerate(self.active)]

        if not self.hide_inactive:
            items += [("pack", CaseRow(c, (i+1+len(self.active))*self.Id, self.lengths, active=False))
                    for i, c in enumerate(self.inactive)]

        return urwid.BoxAdapter(urwid.Frame(
            header=urwid.Text(("casefolder", self.props_str)),
            body=urwid.Pile(items),
            footer=urwid.Divider("─")),
            height=len(items) + 2) # items + 1 and header and footer

    @property
    def props_str(self):
        num_active = len(self.elems["active"])
        num_inactive = len(self.elems["inactive"])
        return "Folder: {} total: {}, active: {}".format(
                self.path, num_inactive + num_active, num_active)

    def update(self):
        self._w = self.draw()
        return self

class CasesListFrame():


    def __init__(self, cases, hide_inactive):
        self.cases = cases
        self.hide_inactive = hide_inactive

    def draw(self):
        """ return a ListBox with all sub folder """
        lengths, valid_cases = self.cases.get_valid_cases()
        items = [urwid.Text(TableHeader(lengths).header_text)]
        items += [DisplaySub(i+1, path, elems, lengths, self.hide_inactive)
            for i, (path, elems) in enumerate(valid_cases.items())]
        return urwid.ListBox(urwid.SimpleFocusListWalker(items))

    def toggle_hide(self):
        self.hide_inactive = not self.hide_inactive


class ScreenParent(urwid.WidgetWrap):

    def __init__(self, frame, mode_switch):

        self._w = frame
        self.input_mode = False
        self.mode_switch = mode_switch
        self.input_txt = ""
        urwid.WidgetWrap.__init__(self, self._w)

    def update(self):
        self._w = self.draw()
        return self

    def keypress_parent(self, size, key):
        global FOCUS_ID
        global FILTER
        if key == 'Q' or key == 'q':
            self.cases.running = False
            raise urwid.ExitMainLoop()
        elif self.input_mode:
            if key != "enter" and key != "backspace":
                self.input_txt += key
                self.input_mode_footer_txt += key
                self._w = self.draw()
            elif key == "backspace":
                self.input_txt = self.input_txt[0:-1]
                self.input_mode_footer_txt = self.input_mode_footer_txt[0:-1]
                self._w = self.draw()
            elif "enter" in key and self.input_mode == "Focus":
                # self.focus_mode = True
                FOCUS_ID = self.input_txt
                global MODE_SWITCH
                MODE_SWITCH = True
                FILTER = None
                self.input_mode = False
            elif "enter" in key and self.input_mode == "Filter":
                # self.focus_mode = True
                FILTER = self.input_txt
                self.input_mode = False


class OverviewScreen(ScreenParent):

    def __init__(self, cases, focus_id, mode_switch, hide_inactive=False):
        self.cases = cases
        self.focus_id = focus_id
        self.hide_inactive = hide_inactive
        self.cases_list_frame = CasesListFrame(self.cases, self.hide_inactive)
        self.input_mode_footer_txt = "Case ID: "

        # Draw empty screen first to construct base class
        self._w = urwid.Text("")
        self.mode_switch = mode_switch
        ScreenParent.__init__(self, self._w, self.mode_switch)
        self._w = self.draw()

    @property
    def footer(self):
        if not self.input_mode:
            menu = urwid.Text([
                    u'Press (', ('mode button', u'T'), u') to toggle active, ',
                    u'(', ('mode button', u'F'), u') to focus, ',
                    u'(', ('quit button', u'Q'), u') to quit,'],
                        align="right")
            legend = urwid.Text(["Legend: ",
                ("progress", " "), " Progress ",
                ("active", "Active"), " ",
                ("inactive", "Inactive"), " ",
                ("sampling", " "), " Sampling Start"])
            return urwid.Columns([legend, menu])
        else:
            return urwid.Edit(self.input_mode_footer_txt)

    def draw(self):

        banner = urwid.Text(foamMonHeader, "center")
        body = urwid.LineBox(self.cases_list_frame.draw())
        footer = self.footer

        return urwid.Frame(header=banner, body=body, footer=footer)

    def keypress(self, size, key):
        if key == 'F' or key == 'f':
            self.input_mode = "Focus"
            self._w = self.draw()
        elif key == 'T' or key == 't':
            self.cases_list_frame.toggle_hide()
            self._w = self.draw()
        else:
            self.keypress_parent(size, key)


class FocusScreen(ScreenParent):

    def __init__(self, focus_id):
        self.focus_id = focus_id
        self.hide_inactive = False
        self.input_mode_footer_txt = "Filter: "
        self._w = urwid.Text("")
        ScreenParent.__init__(self, self._w, False)
        self._w = self.draw()

    def draw(self):

        global CASE_REFS
        global FOCUS_ID
        banner = urwid.Text(foamMonHeader, "center")
        # body = urwid.LineBox(self.cases_list_frame.draw())
        global FILTER
        body = urwid.Pile([
            ("pack", urwid.Text(CASE_REFS[int(FOCUS_ID)].path)),
            ("pack", urwid.Text(CASE_REFS[int(FOCUS_ID)].log.text(FILTER)))])
        footer = self.footer

        return urwid.Frame(header=banner, body=body, footer=footer)


    @property
    def footer(self):
        if not self.input_mode:
            menu = urwid.Text([
                    u'Press (', ('mode button', u'O'), u') for overview mode, ',
                    u'(', ('mode button', u'/'), u') to filter, ',
                    u'(', ('quit button', u'Q'), u') to quit,'],
                        align="right")
            legend = urwid.Text(["Legend: ",
                ("progress", " "), " Progress ",
                ("active", "Active"), " ",
                ("inactive", "Inactive"), " ",
                ("sampling", " "), " Sampling Start"])
            return urwid.Columns([legend, menu])
        else:
            return urwid.Edit(self.input_mode_footer_txt)

    def keypress(self, size, key):
        global IN
        if key == '/':
            self.input_mode = "Filter"
            self._w = self.draw()
        elif key == 'O' or key == 'o' and not self.input_mode:
            global MODE_SWITCH
            MODE_SWITCH = True
        else:
            self.keypress_parent(size, key)


class LogMonFrame(urwid.WidgetWrap):

    def __init__(self, cases):
        self.cases = cases
        self.focus_mode = False
        self.focus_id = None
        self.mode_switch = False
        self.frame = OverviewScreen(self.cases, self.focus_id, self.mode_switch)
        self._w = self.frame
        urwid.WidgetWrap.__init__(self, self._w)

    def draw(self):
        """ returns either a FocusScreen or OverviewScreen instance """
        global MODE_SWITCH
        if not MODE_SWITCH:
            self.frame = self.frame.update()
            return self.frame
        else:
            if isinstance(self.frame, OverviewScreen):
                self.frame = FocusScreen(self.focus_id)
                MODE_SWITCH = False
                return self.frame
                # self._w = self.frame
            else:
                self.frame = OverviewScreen(self.cases, self.focus_id, self.mode_switch)
                MODE_SWITCH = False
                return self.frame

    def keypress(self, size, key):
        """ delegates keypress to the actual screen """
        self._w.keypress(size, key)

    def animate(self, loop=None, data=None):
        # Reset the case display number counter
        global  CASE_CTR
        CASE_CTR = 0

        self.frame = self.draw() # bodyTxt.update()
        self._w = self.frame
        self.animate_alarm = self.loop.set_alarm_in(1.0/30.0, self.animate)

def cui_main():

    cases = Cases(os.getcwd())

    frame = LogMonFrame(cases)
    mainloop = urwid.MainLoop(frame, palette)
    mainloop.screen.set_terminal_properties(colors=256)
    frame.loop = mainloop
    frame.animate()
    mainloop.run()

